﻿using System.Collections.Generic;

using Newtonsoft.Json;
using RestSharp;

namespace ApiNopCommerceSync.Nop
{
    public class ApiClient
    {
        private string url;
        private string client_id;
        private string client_secret;

        public ApiClient(string url, string client_id, string client_secret)
        {
            this.url = url;
            this.client_id = client_id;
            this.client_secret = client_secret;
        }

        public string AuthorizeClient()
        {
            var restclient = new RestClient(url);
            RestRequest request = new RestRequest("/oauth/authorize") { Method = Method.GET };
            request.AddHeader("Accept", "application/json");
            request.AddHeader("Content-Type", "application/x-www-form-urlencoded");
            request.AddParameter("client_id", client_id);
            request.AddParameter("client_secret", client_secret);
            request.AddParameter("response_type", "code");
            var tResponse = restclient.Execute(request);
            var code = tResponse.ResponseUri.Query.Replace("?code=", "");

            request = new RestRequest("/api/token") { Method = Method.POST };
            request.AddHeader("Accept", "application/json");
            request.AddHeader("Content-Type", "application/x-www-form-urlencoded");
            request.AddParameter("client_id", client_id);
            request.AddParameter("client_secret", client_secret);
            request.AddParameter("code", code);
            request.AddParameter("grant_type", "authorization_code");
            tResponse = restclient.Execute(request);
            var responseJson = tResponse.Content;
            var token = JsonConvert.DeserializeObject<Dictionary<string, object>>(responseJson)["access_token"].ToString();
            
            return token;
        }
        
        public Dictionary<string, object> Get(string resource, string token, Dictionary<string, string> headers = default(Dictionary<string, string>), Dictionary<string, string> parameters = default(Dictionary<string,string>))
        {
            var restclient = new RestClient(url);
            RestRequest request = new RestRequest(resource) { Method = Method.GET };
            request.AddHeader("Authorization", string.Format("Bearer {0}", token));
            if (headers != default(Dictionary<string, string>))
            {
                foreach (var item in headers)
                {
                    request.AddHeader(item.Key, item.Value);
                }
            }
            if (parameters != default(Dictionary<string, string>))
            {
                foreach (var item in parameters)
                {
                    request.AddHeader(item.Key, item.Value);
                }
            }
            var tResponse = restclient.Execute(request);
            var responseJson = tResponse.Content;
            var result = JsonConvert.DeserializeObject<Dictionary<string, object>>(responseJson);
            return result;
        }

        public Dictionary<string, object> Post(string resource, string token, Dictionary<string, string> headers = default(Dictionary<string, string>), Dictionary<string, string> parameters = default(Dictionary<string, string>), object body = null)
        {
            var restclient = new RestClient(url);
            RestRequest request = new RestRequest(resource) { Method = Method.POST };
            request.RequestFormat = DataFormat.Json;
            request.AddHeader("Authorization", string.Format("Bearer {0}", token));
            request.AddHeader("Accept", "application/json");
            //request.AddHeader("Content-Type", "application/x-www-form-urlencoded");
            
            if (body != null)
                request.AddParameter("application/json", body, ParameterType.RequestBody); //request.AddBody(body);
            if (headers != default(Dictionary<string, string>))
            {
                foreach (var item in headers)
                {
                    request.AddHeader(item.Key, item.Value);
                }
            }
            if (parameters != default(Dictionary<string, string>))
            {
                foreach (var item in parameters)
                {
                    request.AddHeader(item.Key, item.Value);
                }
            }
            var tResponse = restclient.Execute(request);
            var responseJson = tResponse.Content;
            var result = JsonConvert.DeserializeObject<Dictionary<string, object>>(responseJson);
            return result;
        }

        public Dictionary<string, object> Put(string resource, string token, Dictionary<string, string> headers = default(Dictionary<string, string>), Dictionary<string, string> parameters = default(Dictionary<string, string>), object body = null)
        {
            var restclient = new RestClient(url);
            RestRequest request = new RestRequest(resource) { Method = Method.PUT };
            request.RequestFormat = DataFormat.Json;
            request.AddHeader("Authorization", string.Format("Bearer {0}", token));
            request.AddHeader("Accept", "application/json");
            request.AddHeader("Content-Type", "application/x-www-form-urlencoded");
            if (body != null)
                request.AddBody(body);
            if (headers != default(Dictionary<string, string>))
            {
                foreach (var item in headers)
                {
                    request.AddHeader(item.Key, item.Value);
                }
            }
            if (parameters != default(Dictionary<string, string>))
            {
                foreach (var item in parameters)
                {
                    request.AddHeader(item.Key, item.Value);
                }
            }
            var tResponse = restclient.Execute(request);
            var responseJson = tResponse.Content;
            var result = JsonConvert.DeserializeObject<Dictionary<string, object>>(responseJson);
            return result;
        }
    }
}
